#!/usr/bin/env python2.3
######################################################################
#
#   Module:  libtile
#   Version: $Id: libtile.py,v 1.2 2003/09/03 16:53:15 jsc Exp $
#   Author:  Jeffrey Clement <jclement@bluesine.com>
#   Targets: Win32, Unix
#   Web:     http://jclement.ca/
#
# Library to handle tiling of one image into a single image with 
# borders and stuff like that.  Suitable for embedding into other 
# programs.  Using the Python Imaging Library (PIL).
#
# -------------------------------------------------------------------
#
# $Log: libtile.py,v $
# Revision 1.2  2003/09/03 16:53:15  jsc
# Fixes to documentation
#
# -------------------------------------------------------------------
#
# Copyright (c) 2003, Jeffrey Clement All rights reserved. 
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions 
# are met: 
#
# * Redistributions of source code must retain the above copyright notice, 
#   this list of conditions and the following disclaimer. 
# * Redistributions in binary form must reproduce the above copyright 
#   notice, this list of conditions and the following disclaimer in the 
#   documentation and/or other materials provided with the distribution. 
# * Neither the name of the Bluesine nor the names of its contributors 
#   may be used to endorse or promote products derived from this software 
#   without specific prior written permission. 
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED 
# TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
# PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR 
# CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
##########################################################################

import Image
import math

def resize(size, hwRatio):
    """
    Return the minimum size of image that is at least "size" but
    maintains the provided aspect ratio
    """
    if size[0] * hwRatio > size[1]:
        return (size[0], int(math.ceil(size[0]*hwRatio)))
    else:
        return (int(math.ceil(1.0*size[1] / hwRatio)), size[1])
    

def tile(dimensions, src, hwRatio=0.6667, paddingPercent=0.05, 
         borderBg=(255,255,255), pageBg=(0,0,0), outsideBorderPercent=0.1):
    """
    Tile an image and return a new image with the resulting tiled image
    with borders and stuff like that.

    dimensions: A tuple in form (width, height) that indicates the 
        number of tiles horizontally and vertically in the output image.
        
    src: A PIL Image object with the source image you wish to tile

    hwRatio: Ratio of height to width required in output image.  Lets
        you create an image with exactly a 4x6 ratio for printing to
        online photo services.

    paddingPercent: Poorly named but this is the border that surrounds
        each image expressed as a fraction of it's width!

    borderBg: a tuple in the form (r,g,b) that provides the border 
        color for the images.  ie) you will have a border of this 
        color at a width of paddingPercent * sourceImage.width 
        around each image.

    pageBg: a tuple in the form (r,g,b) that sets the color for 
        the background image.  This will be seen between images
        as well as around the entire image.

    outsideBorderPercent: width of outside border as fraction of
        output image width and height.  This allows for extra 
        padding to compensate for poor cropping by photo finishers.

    EG:
        # this tiles an image 2x3 with a 5% red border and saves it
        import Image
        src = Image.load('test.jpg')
        dest = tile((2,3), src, 4.0/6, 0.05, (255,0,0))
        dest.save('test-tiled.jpg')
        
    """
    nw, nh = dimensions
    iw, ih = src.size
    pp = paddingPercent
    # padding is a fraction of the source image size
    pah = int(math.ceil(pp * iw))
    paw = int(math.ceil(pp * iw))
    # optimal image dimensions
    # img = B1+B2+IMG+B2+B1+B2+IMG+B2+B1
    niw = iw * nw + paw * (4+3*(nw-1))
    nih = ih * nh + pah * (4+3*(nh-1))
    # compute min image size that has the correct AR
    fw, fh = resize((niw, nih), hwRatio)
    print niw,fw, nih,fh
    # outside border as fraction of the total size
    fw = int(fw * (1+ outsideBorderPercent));
    fh = int(fh * (1+ outsideBorderPercent));
    # calculate offsets for the image to center the tiles in the output image
    woff, hoff = int((fw-niw)/2.0), int((fh-nih)/2.0)
    dest = Image.new('RGB', (fw,fh),pageBg)
    border = Image.new('RGB', (iw+2*paw, ih+2*pah),borderBg)
    for w in range(nw):
        for h in range(nh):
            dx = paw + woff + w * (iw + paw*3)
            dy = pah + hoff + h * (ih + pah*3)
            dest.paste(border, (dx, dy))
            dest.paste(src, (dx+paw, dy+pah))
    return dest

if __name__=='__main__':
    print "Copyright (C) 2003 Jeffrey Clement"
    print "This is free software; see the source for copying conditions.  There is NO"
    print "warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE."
    print ""
    print "(view the source code for details on using this library)"

